@@ -12,12 +12,12 @@ from sarus_data_spec.variant_constraint import VariantConstraint
 # pylint: disable=unused-variable
 
 PACKAGE_NAME: Final[str] = 'sarus_data_spec'
-VERSION: Final[str] = '1.3.17'
+VERSION: Final[str] = '2.1.0'
 
 try:
     # The local context is absent from the public release
-    import sarus_data_spec.context.local as sc
+    import sarus_data_spec.context.worker as sw
 
-    push_global_context(sc.Local())
+    push_global_context(sw.WorkerContext())
 except ModuleNotFoundError:
     pass
