@@ -34,9 +34,17 @@ try:
 except ModuleNotFoundError:
     warnings.warn('SqlAlchemy not found, sql operations not available')
 
+try:
+    from sarus_differential_privacy.protobuf.private_query_pb2 import (
+        PrivateQuery,
+    )
+except ModuleNotFoundError:
+    pass  # warning raised in typing.py
+
+
 from sarus_data_spec.base import Referring
-from sarus_data_spec.constants import DATASET_SLUGNAME
-from sarus_data_spec.protobuf.utilities import to_base64
+from sarus_data_spec.constants import DATASET_SLUGNAME, PRIVATE_QUERY
+from sarus_data_spec.protobuf.utilities import dejson, to_base64
 from sarus_data_spec.scalar import Scalar
 from sarus_data_spec.transform import Transform
 import sarus_data_spec.protobuf as sp
@@ -45,16 +53,15 @@ import sarus_data_spec.typing as st
 
 if TYPE_CHECKING:
     from sarus_data_spec.bounds import Bounds
+    from sarus_data_spec.links import Links
     from sarus_data_spec.marginals import Marginals
-    from sarus_data_spec.schema import Schema
     from sarus_data_spec.size import Size
-    from sarus_data_spec.status import Status
 
 
 class Dataset(Referring[sp.Dataset]):
     """A python class to describe datasets"""
 
-    def __init__(self, protobuf: sp.Dataset) -> None:
+    def __init__(self, protobuf: sp.Dataset, store: bool = True) -> None:
         if protobuf.spec.HasField("transformed"):
             transformed = protobuf.spec.transformed
             self._referred = {
@@ -63,7 +70,7 @@ class Dataset(Referring[sp.Dataset]):
                 *list(transformed.named_arguments.values()),
             }
 
-        super().__init__(protobuf=protobuf)
+        super().__init__(protobuf=protobuf, store=store)
 
     def prototype(self) -> Type[sp.Dataset]:
         """Return the type of the underlying protobuf."""
@@ -93,14 +100,19 @@ class Dataset(Referring[sp.Dataset]):
 
     def is_protected(self) -> bool:
         """Is the dataset protected."""
-        if self.is_transformed():
-            transform = self.transform()
-            return transform.protobuf().spec.WhichOneof('spec') in [
-                "protect_dataset",
-                'user_settings',
-            ]
-        else:
-            return False
+        return self.schema().is_protected()
+
+    def is_pep(self) -> bool:
+        """Is the dataset PEP."""
+        return self.pep_token() is not None
+
+    def pep_token(self) -> Optional[str]:
+        """Returns the dataset PEP token."""
+        return self.manager().query_manager().pep_token(self)
+
+    def is_public(self) -> bool:
+        """Is the dataset public."""
+        return self.manager().query_manager().is_public(self)
 
     def is_remote(self) -> bool:
         """Is the dataspec a remotely defined dataset."""
@@ -148,21 +160,40 @@ class Dataset(Referring[sp.Dataset]):
         self.accept(visitor)
         return visitor.results
 
-    def status(self) -> Status:
-        return cast('Status', self.manager().status(self))
+    def status(self, task_names: t.List[str]) -> t.Optional[st.Status]:
+        """This method return a status that contains all the
+        last updates for the task_names required. It returns None if
+        all the tasks are missing. Synchronization is performed under the
+        hood, so statuses with the task_names are copied in the storage if
+        the current manager has a delegated."""
 
-    def schema(self) -> Schema:
-        return cast('Schema', self.manager().schema(self))
+        # collects last status for each task. This if needed will also update
+        # them if any synchronization is needed.
+        statuses = [
+            self.manager().status(self, task_name) for task_name in task_names
+        ]
+        # go from last to first so to be sure that tasks have been updated
+        # for all others if needed
+        for status in reversed(statuses):
+            if status is not None:
+                return status
+        return None
 
-    def size(self) -> Size:
+    def schema(self) -> st.Schema:
+        return self.manager().schema(self)
+
+    def size(self) -> t.Optional[Size]:
         return cast('Size', self.manager().size(self))
 
-    def bounds(self) -> Bounds:
+    def bounds(self) -> t.Optional[Bounds]:
         return cast('Bounds', self.manager().bounds(self))
 
-    def marginals(self) -> Marginals:
+    def marginals(self) -> t.Optional[Marginals]:
         return cast('Marginals', self.manager().marginals(self))
 
+    def links(self) -> st.Links:
+        return cast('Links', self.manager().links(self))
+
     def transform(self) -> st.Transform:
         return cast(
             st.Transform,
@@ -171,7 +202,9 @@ class Dataset(Referring[sp.Dataset]):
             ),
         )
 
-    def to_arrow(self, batch_size: int = 10000) -> Iterator[pa.RecordBatch]:
+    def to_arrow(
+        self, batch_size: int = 10000
+    ) -> st.ContextManagerIterator[pa.RecordBatch]:
         return self.manager().to_arrow(self, batch_size)
 
     async def async_to_arrow(
@@ -199,44 +232,40 @@ class Dataset(Referring[sp.Dataset]):
 
         return args_parents, kwargs_parents
 
-    def is_compliant(
-        self,
-        kind: st.ConstraintKind,
-        public_context: List[str],
-        epsilon: Optional[float] = None,
-    ) -> bool:
-        return self.manager().is_compliant(self, kind, public_context, epsilon)
-
     def variant(
         self,
         kind: st.ConstraintKind,
-        public_context: List[str],
-        epsilon: Optional[float] = None,
+        public_context: List[str] = [],
+        privacy_limit: Optional[st.PrivacyLimit] = None,
     ) -> Optional[st.DataSpec]:
-        return self.manager().variant(self, kind, public_context, epsilon)
-
-    def variant_constraint(self) -> Optional[st.VariantConstraint]:
-        return self.manager().variant_constraint(self)
+        return (
+            self.manager()
+            .query_manager()
+            .variant(self, kind, public_context, privacy_limit)
+        )
 
     def variants(self) -> Collection[st.DataSpec]:
-        return self.manager().variants(self)
+        return self.manager().query_manager().variants(self)
 
-    def verifies(
-        self,
-        kind: st.ConstraintKind,
-        public_context: Collection[str],
-        epsilon: Optional[float],
-    ) -> bool:
-        variant_constraint = self.variant_constraint()
-        if variant_constraint:
-            return self.manager().verifies(
-                variant_constraint=variant_constraint,
-                kind=kind,
-                public_context=public_context,
-                epsilon=epsilon,
-            )
-        else:
-            return False
+    def private_queries(self) -> List[PrivateQuery]:
+        """Return the list of PrivateQueries used in a Dataspec's transform.
+
+        It represents the privacy loss associated with the current computation.
+
+        It can be used by Sarus when a user (Access object) reads a DP dataspec
+        to update its accountant. Note that Private Query objects are generated
+        with a random uuid so that even if they are submitted multiple times to
+        an account, they are only accounted once (ask @cgastaud for more on
+        accounting)."""
+        attribute = self.attribute(name=PRIVATE_QUERY)
+        if attribute is None:
+            return []
+
+        private_query_str = attribute[PRIVATE_QUERY]
+        return [
+            cast(PrivateQuery, dejson(q))
+            for q in json.loads(private_query_str)
+        ]
 
     def spec(self) -> str:
         return cast(str, self.protobuf().spec.WhichOneof('spec'))
@@ -346,6 +375,9 @@ class Dataset(Referring[sp.Dataset]):
     def primary_keys(self) -> List[st.Path]:
         return self.manager().primary_keys(self)
 
+    def attribute(self, name: str) -> t.Optional[st.Attribute]:
+        return self.manager().attribute(name=name, dataspec=self)
+
 
 # Builders
 def transformed(
@@ -385,12 +417,18 @@ def transformed(
         )
     else:
         properties = {}
-        if len(arguments) + len(named_arguments) == 1:
-            (unique_arg,) = list(arguments) + list(named_arguments.values())
-            if DATASET_SLUGNAME in unique_arg.properties():
-                properties[DATASET_SLUGNAME] = unique_arg.properties()[
-                    DATASET_SLUGNAME
-                ]
+        ds_args = [
+            element
+            for element in arguments
+            if element.type_name() == sp.type_name(sp.Dataset)
+        ]
+        for element in named_arguments.values():
+            if element.type_name() == sp.type_name(sp.Dataset):
+                ds_args.append(element)
+        if len(ds_args) == 1 and DATASET_SLUGNAME in ds_args[0].properties():
+            properties[DATASET_SLUGNAME] = arguments[0].properties()[
+                DATASET_SLUGNAME
+            ]
 
         output_dataspec = Dataset(
             sp.Dataset(
@@ -557,5 +595,5 @@ if t.TYPE_CHECKING:
     test_files: st.Dataset = files(name='', uri_pattern='', format='')
     test_csv_files: st.Dataset = csv_files(name='', uri_pattern='')
     test_transformed: st.DataSpec = transformed(
-        sdtr.protect(protected_entities=[]), sql(uri='sqlite:///:memory:')
+        sdtr.protect(), sql(uri='sqlite:///:memory:')
     )
