@@ -1,7 +1,6 @@
 from __future__ import annotations
 
 import hashlib
-import json
 import os
 import typing as t
 import warnings
@@ -23,14 +22,17 @@ from sarus_data_spec.constants import (
     DATASET_N_BYTES,
     DATASET_N_LINES,
     IS_BIG_DATA,
+    IS_REMOTE,
     THRESHOLD_TYPE,
 )
 from sarus_data_spec.protobuf.utilities import copy
 from sarus_data_spec.protobuf.utilities import json as utilities_json
 from sarus_data_spec.protobuf.utilities import serialize, type_name
 from sarus_data_spec.query_manager.base import BaseQueryManager
+from sarus_data_spec.scalar import Scalar
 import sarus_data_spec.manager.typing as manager_typing
 import sarus_data_spec.protobuf as sp
+import sarus_data_spec.query_manager.typing as sqmt
 import sarus_data_spec.status as stt
 import sarus_data_spec.storage.typing as storage_typing
 import sarus_data_spec.typing as st
@@ -48,7 +50,10 @@ class Base(manager_typing.Manager):
         self.storage().store(self)
         self._parquet_dir = os.path.expanduser('/tmp/sarus_dataset/')
         os.makedirs(self.parquet_dir(), exist_ok=True)
-        self.query_manager = BaseQueryManager(storage=storage)
+        self._query_manager = BaseQueryManager(storage=storage)
+
+    def query_manager(self) -> sqmt.QueryManager:
+        return self._query_manager
 
     def parquet_dir(self) -> str:
         return self._parquet_dir
@@ -111,57 +116,21 @@ class Base(manager_typing.Manager):
     def size(self, dataset: st.Dataset) -> st.Size:
         raise NotImplementedError
 
-    def is_compliant(
-        self,
-        dataspec: st.DataSpec,
-        kind: st.ConstraintKind,
-        public_context: t.List[str],
-        epsilon: t.Optional[float],
-    ) -> bool:
-        return self.query_manager.is_compliant(
-            dataspec,
-            kind=kind,
-            public_context=public_context,
-            epsilon=epsilon,
-        )
-
-    def variant(
-        self,
-        dataspec: st.DataSpec,
-        kind: st.ConstraintKind,
-        public_context: t.List[str],
-        epsilon: t.Optional[float],
-    ) -> t.Optional[st.DataSpec]:
-        return self.query_manager.variant(
-            dataspec=dataspec,
-            kind=kind,
-            public_context=public_context,
-            epsilon=epsilon,
-        )
-
-    def variants(self, dataspec: st.DataSpec) -> t.Collection[st.DataSpec]:
-        return self.query_manager.variants(dataspec=dataspec)
-
-    def variant_constraint(
-        self, dataspec: st.DataSpec
-    ) -> t.Optional[st.VariantConstraint]:
-        return self.query_manager.variant_constraint(dataspec)
-
     def set_remote(self, dataspec: st.DataSpec) -> None:
         """Add an Attribute to tag the DataSpec as remotely fetched."""
-        attach_properties(dataspec, {"is_remote": str(True)})
+        attach_properties(
+            dataspec,
+            properties={
+                # TODO deprecated in SDS >=2.0.0 -> remove property
+                "is_remote": str(True)
+            },
+            name=IS_REMOTE,
+        )
 
     def is_remote(self, dataspec: st.DataSpec) -> bool:
         """Is the dataspec a remotely defined dataset."""
-        attributes = self.storage().referring(
-            dataspec, type_name=sp.type_name(sp.Attribute)
-        )
-        is_remote = [
-            att.properties()["is_remote"]
-            for att in attributes
-            if "is_remote" in att.properties()
-        ]
-        return str(True) in is_remote
+        is_remote_att = self.attribute(IS_REMOTE, dataspec)
+        return is_remote_att is not None
 
     def infer_output_type(
         self,
@@ -174,35 +143,30 @@ class Base(manager_typing.Manager):
         def attach_nothing(ds: st.DataSpec) -> None:
             return
 
-        if not transform.is_external():
+        if transform.is_external() or transform.spec() in [
+            'protected_paths',
+            'automatic_user_settings',
+            'public_paths',
+            'automatic_budget',
+            'attribute_budget',
+            'sd_budget',
+            'sampling_ratios',
+            'derive_seed',
+            'relationship_spec',
+        ]:
             # By default, results of non external transforms (e.g. join,
             # sample) are Datasets and non external transforms are only applied
             # to Datasets
+            return sp.type_name(sp.Scalar), attach_nothing
 
-            return sp.type_name(sp.Dataset), attach_nothing
-
-        return sp.type_name(sp.Scalar), attach_nothing
-
-    def verifies(
-        self,
-        variant_constraint: st.VariantConstraint,
-        kind: st.ConstraintKind,
-        public_context: t.Collection[str],
-        epsilon: t.Optional[float],
-    ) -> bool:
-        return self.query_manager.verifies(
-            variant_constraint=variant_constraint,
-            kind=kind,
-            public_context=public_context,
-            epsilon=epsilon,
-        )
+        return sp.type_name(sp.Dataset), attach_nothing
 
     def bounds(self, dataset: st.Dataset) -> st.Bounds:
         raise NotImplementedError
 
     def to_arrow(
         self, dataset: st.Dataset, batch_size: int
-    ) -> t.Iterator[pa.RecordBatch]:
+    ) -> st.ContextManagerIterator[pa.RecordBatch]:
         raise NotImplementedError
 
     def to_pandas(self, dataset: st.Dataset) -> pd.DataFrame:
@@ -211,6 +175,9 @@ class Base(manager_typing.Manager):
     def to_parquet(self, dataset: st.Dataset) -> None:
         raise NotImplementedError
 
+    async def async_to_parquet(self, dataset: st.Dataset) -> None:
+        raise NotImplementedError
+
     def to_tensorflow(self, dataset: st.Dataset) -> tf.data.Dataset:
         raise NotImplementedError
 
@@ -220,6 +187,12 @@ class Base(manager_typing.Manager):
     async def async_value(self, scalar: st.Scalar) -> st.DataSpecValue:
         raise NotImplementedError
 
+    def cache_scalar(self, scalar: st.Scalar) -> None:
+        raise NotImplementedError
+
+    async def async_cache_scalar(self, scalar: st.Scalar) -> None:
+        raise NotImplementedError
+
     async def async_to_arrow(
         self, dataset: st.Dataset, batch_size: int
     ) -> t.AsyncIterator[pa.RecordBatch]:
@@ -233,13 +206,23 @@ class Base(manager_typing.Manager):
     ) -> tf.data.Dataset:
         raise NotImplementedError
 
-    def status(self, dataset: st.DataSpec) -> st.Status:
-        result = self.storage().last_referring(
-            [self, dataset], type_name='Status'
+    async def async_bounds(self, dataset: st.Dataset) -> st.Bounds:
+        raise NotImplementedError
+
+    async def async_size(self, dataset: st.Dataset) -> st.Size:
+        raise NotImplementedError
+
+    async def async_marginals(self, dataset: st.Dataset) -> st.Marginals:
+        raise NotImplementedError
+
+    def status(
+        self, dataspec: st.DataSpec, task_name: str
+    ) -> t.Optional[st.Status]:
+        return stt.last_status(
+            dataspec=dataspec,
+            manager=self,
+            task=task_name,
         )
-        if result is None:
-            raise RuntimeWarning("No status.")
-        return t.cast(st.Status, result)
 
     def sql(
         self,
@@ -257,9 +240,9 @@ class Base(manager_typing.Manager):
 
     def is_big_data(self, dataset: st.Dataset) -> bool:
 
-        statuses = stt.last_statuses(dataset, BIG_DATA_TASK)
-        if len(statuses) > 0:
-            status = statuses[0]
+        status = stt.last_status(dataset, task=BIG_DATA_TASK)
+
+        if status is not None:
             # check if big_data_present
             big_data_task = status.task(BIG_DATA_TASK)
             # if yes:return answer
@@ -274,15 +257,24 @@ class Base(manager_typing.Manager):
             )
         else:
             parents_list, parents_dict = dataset.parents()
-            parents_list.extend(list(parents_dict.values()))
+            ds_args = [
+                element
+                for element in parents_list
+                if element.type_name() == sp.type_name(sp.Dataset)
+            ]
+            for element in parents_dict.values():
+                if element.type_name() == sp.type_name(sp.Dataset):
+                    ds_args.append(element)
+            # parents_list.extend(list(parents_dict.values()))
             # parents_list=t.cast(t.Sequence[st.Dataset],parents_list)
-            if len(parents_list) > 1:
+            if len(ds_args) > 1:
                 raise NotImplementedError(
                     'transforms with many dataspecs not supported yet'
                 )
             is_parent_big_data = self.is_big_data(
-                parents_list[0]  # type:ignore
+                ds_args[0]  # type:ignore
             )
+
             if not is_parent_big_data:
                 # write status it is not big data
                 stt.ready(
@@ -303,10 +295,14 @@ class Base(manager_typing.Manager):
                     number_bytes,
                     threshold_kind,
                 ) = check_transform_big_data(
-                    dataset.transform(), parents_list[0]  # type:ignore
+                    dataset.transform(),
+                    parents_list[0],  # type:ignore
+                    parents_dict,
                 )
                 big_data_threshold = int(
-                    stt.last_statuses(parents_list[0])[0]  # type:ignore
+                    stt.last_status(
+                        dataspec=parents_list[0], task=BIG_DATA_TASK
+                    )  # type:ignore
                     .task(BIG_DATA_TASK)
                     .properties()[BIG_DATA_THRESHOLD]
                 )
@@ -324,34 +320,69 @@ class Base(manager_typing.Manager):
                 )
                 return is_big_data
 
+    def is_cached(self, dataspec: st.DataSpec) -> bool:
+        raise NotImplementedError
+
+    async def async_primary_keys(self, dataset: st.Dataset) -> t.List[st.Path]:
+        raise NotImplementedError
+
+    async def async_foreign_keys(
+        self, dataset: st.Dataset
+    ) -> t.Dict[st.Path, st.Path]:
+        raise NotImplementedError
+
+    def attribute(
+        self, name: str, dataspec: st.DataSpec
+    ) -> t.Optional[st.Attribute]:
+
+        attributes = t.cast(
+            t.Set[st.Attribute],
+            self.storage().referring(
+                dataspec, type_name=sp.type_name(sp.Attribute)
+            ),
+        )
+        filtered_attributes = [
+            element for element in attributes if element.name() == name
+        ]
+
+        if len(filtered_attributes) == 0:
+            return None
+
+        if len(filtered_attributes) == 1:
+            return t.cast(st.Attribute, filtered_attributes[0])
+
+        raise ValueError('There are two attributes with the same name')
+
+    def links(self, dataset: st.Dataset) -> st.Links:
+        raise NotImplementedError
+
 
 def check_transform_big_data(
-    transform: st.Transform, parent_dataset: st.Dataset
+    transform: st.Transform,
+    parent_dataset: st.Dataset,
+    parents_dict: t.Mapping[str, st.DataSpec],
 ) -> t.Tuple[bool, int, int, str]:
     """This methods return true if the dataset transformed
     is big_data and False otherwise. This method is called when the parent
     is big_data so if the transform does not
     affect the size, it should return True
     """
-    statuses = stt.last_statuses(parent_dataset, BIG_DATA_TASK)
-    assert len(statuses) > 0
-    status = statuses[0]
-    stage = status.task(BIG_DATA_TASK)
-    assert stage
-    big_data_threshold = int(stage.properties()[BIG_DATA_THRESHOLD])
-    threshold_kind = stage.properties()[THRESHOLD_TYPE]
+    status = stt.last_status(parent_dataset, task=BIG_DATA_TASK)
+    big_data_threshold = int(
+        status.task(BIG_DATA_TASK).properties()[  # type:ignore
+            BIG_DATA_THRESHOLD
+        ]
+    )
+    threshold_kind = status.task(BIG_DATA_TASK).properties()[  # type:ignore
+        THRESHOLD_TYPE
+    ]
 
-    parent_n_lines_str = stage.properties()[DATASET_N_LINES]
-    if parent_n_lines_str == '':
-        parent_n_lines = 0
-    else:
-        parent_n_lines = int(parent_n_lines_str)
-
-    parent_bytes_str = stage.properties()[DATASET_N_BYTES]
-    if parent_bytes_str == '':
-        parent_bytes = 0
-    else:
-        parent_bytes = int(parent_bytes_str)
+    parent_n_lines = int(
+        status.task(BIG_DATA_TASK).properties()[DATASET_N_LINES]  # type:ignore
+    )
+    parent_bytes = int(
+        status.task(BIG_DATA_TASK).properties()[DATASET_N_BYTES]  # type:ignore
+    )
 
     transform_name = transform.name()
     if transform_name == 'Sample':
@@ -388,16 +419,16 @@ def check_transform_big_data(
     elif transform_name == 'Synthetic data':
         # here we should leverage the sampling ratio just to get the size,
         # in any case, synthetic data is never big data
-        sampling_ratios = json.loads(transform.properties()['sampling_ratios'])
+        sampling_ratios = t.cast(
+            t.Mapping, t.cast(Scalar, parents_dict['sampling_ratios']).value()
+        )
         synthetic_size = 0
         for table_path, sampling_ratio in zip(
             parent_dataset.schema().tables(), sampling_ratios.values()
         ):
-            stat = (
-                parent_dataset.size()
-                .statistics()
-                .nodes_statistics(table_path)[0]
-            )
+            size = parent_dataset.size()
+            assert size
+            stat = size.statistics().nodes_statistics(table_path)[0]
             synthetic_size += int(stat.size() * sampling_ratio)
         threshold_kind = DATASET_N_LINES
         return (
