@@ -15,61 +15,48 @@ try:
 except ModuleNotFoundError:
     pass  # error message in typing.py
 
-from .utils import sarus_external_op
 
-
-@sarus_external_op
 async def model_fit(model: Any, *args: Any, **kwargs: Any) -> Any:
     fitted_model = model.fit(*args, **kwargs)
     return fitted_model
 
 
-@sarus_external_op
 async def sk_get_n_splits(model: Any, *args: Any, **kwargs: Any) -> Any:
     return model.split(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_inverse_transform(model: Any, *args: Any, **kwargs: Any) -> Any:
     return model.inverse_transform(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_predict(model: Any, *args: Any, **kwargs: Any) -> Any:
     return model.predict(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_predict_log_proba(model: Any, *args: Any, **kwargs: Any) -> Any:
     return model.predict_log_proba(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_predict_proba(model: Any, *args: Any, **kwargs: Any) -> Any:
     return model.predict_proba(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_split(model: Any, *args: Any, **kwargs: Any) -> Any:
     return model.split(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_score(model: Any, *args: Any, **kwargs: Any) -> Any:
     return model.score(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_transform(model: Any, *args: Any, **kwargs: Any) -> Any:
     return model.transform(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_scale(val: Any, *args: Any, **kwargs: Any) -> Any:
     return preprocessing.scale(val, *args, **kwargs)
 
 
-@sarus_external_op
 async def sk_cross_val_score(
     *args: Any,
     **kwargs: Any,
@@ -77,7 +64,6 @@ async def sk_cross_val_score(
     return model_selection.cross_val_score(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_train_test_split(
     *args: Any,
     **kwargs: Any,
@@ -86,7 +72,8 @@ async def sk_train_test_split(
 
 
 # metrics
-@sarus_external_op
+
+
 async def sk_accuracy_score(
     *args: Any,
     **kwargs: Any,
@@ -94,7 +81,6 @@ async def sk_accuracy_score(
     return metrics.accuracy_score(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_average_precision_score(
     *args: Any,
     **kwargs: Any,
@@ -102,7 +88,6 @@ async def sk_average_precision_score(
     return metrics.average_precision_score(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_classification_report(
     *args: Any,
     **kwargs: Any,
@@ -110,7 +95,6 @@ async def sk_classification_report(
     return metrics.classification_report(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_confusion_matrix(
     *args: Any,
     **kwargs: Any,
@@ -118,7 +102,6 @@ async def sk_confusion_matrix(
     return metrics.confusion_matrix(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_f1_score(
     *args: Any,
     **kwargs: Any,
@@ -126,7 +109,6 @@ async def sk_f1_score(
     return metrics.f1_score(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_plot_confusion_matrix(
     *args: Any,
     **kwargs: Any,
@@ -134,7 +116,6 @@ async def sk_plot_confusion_matrix(
     return metrics.plot_confusion_matrix(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_precision_recall_curve(
     *args: Any,
     **kwargs: Any,
@@ -142,7 +123,6 @@ async def sk_precision_recall_curve(
     return metrics.precision_recall_curve(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_precision_score(
     *args: Any,
     **kwargs: Any,
@@ -150,7 +130,6 @@ async def sk_precision_score(
     return metrics.precision_score(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_recall_score(
     *args: Any,
     **kwargs: Any,
@@ -158,7 +137,6 @@ async def sk_recall_score(
     return metrics.recall_score(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_roc_auc_score(
     *args: Any,
     **kwargs: Any,
@@ -166,7 +144,6 @@ async def sk_roc_auc_score(
     return metrics.roc_auc_score(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_auc(
     *args: Any,
     **kwargs: Any,
@@ -174,7 +151,6 @@ async def sk_auc(
     return metrics.auc(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_roc_curve(
     *args: Any,
     **kwargs: Any,
@@ -182,7 +158,6 @@ async def sk_roc_curve(
     return metrics.roc_curve(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_plot_roc_curve(
     *args: Any,
     **kwargs: Any,
@@ -190,7 +165,6 @@ async def sk_plot_roc_curve(
     return metrics.plot_roc_curve(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_plot_precision_recall_curve(
     *args: Any,
     **kwargs: Any,
@@ -198,7 +172,6 @@ async def sk_plot_precision_recall_curve(
     return metrics.plot_precision_recall_curve(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_label_encoder_fit_transform(
     model: Any,
     *args: Any,
@@ -208,7 +181,6 @@ async def sk_label_encoder_fit_transform(
     return X_transformed
 
 
-@sarus_external_op
 async def sk_permutation_importance(
     *args: Any,
     **kwargs: Any,
@@ -216,7 +188,6 @@ async def sk_permutation_importance(
     return inspection.permutation_importance(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_pipeline(
     *args: Any,
     **kwargs: Any,
@@ -224,7 +195,6 @@ async def sk_pipeline(
     return pipeline.Pipeline(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_function_transformer(
     *args: Any,
     **kwargs: Any,
@@ -232,7 +202,6 @@ async def sk_function_transformer(
     return preprocessing.FunctionTransformer(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_column_transformer(
     *args: Any,
     **kwargs: Any,
@@ -240,7 +209,6 @@ async def sk_column_transformer(
     return compose.ColumnTransformer(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_time_series_split(
     *args: Any,
     **kwargs: Any,
@@ -248,7 +216,6 @@ async def sk_time_series_split(
     return model_selection.TimeSeriesSplit(*args, **kwargs)
 
 
-@sarus_external_op
 async def sk_simple_imputer(
     *args: Any,
     **kwargs: Any,
