@@ -1,9 +1,8 @@
 from __future__ import annotations
 
-import json
+import datetime
 import pickle as pkl
 import typing as t
-import warnings
 
 import numpy as np
 
@@ -12,11 +11,6 @@ import sarus_data_spec.dataset as sd
 import sarus_data_spec.protobuf as sp
 import sarus_data_spec.typing as st
 
-try:
-    from sarus_data_spec.path import paths
-except ModuleNotFoundError:
-    warnings.warn('protection not available')
-
 
 class Transform(Referrable[sp.Transform]):
     """A python class to describe transforms"""
@@ -592,24 +586,40 @@ def sample(fraction_size: t.Union[float, int]) -> Transform:
     )
 
 
-def user_settings(user_type: st.Type) -> Transform:
+def user_settings() -> Transform:
+    """Transform to create a dataspec from
+    a protected one with a new schema. It should
+    be called on:
+    - the dataset that needs to be protected as the first arg
+    - a kwarg user_type: scalar output of automatic_user_setttings"""
     return Transform(
         sp.Transform(
-            name='Updated_schema',
-            spec=sp.Transform.Spec(
-                user_settings=sp.Transform.UserSettings(
-                    type=user_type.protobuf()
-                )
-            ),
-            inversible=True,
+            name='User Settings',
+            spec=sp.Transform.Spec(user_settings=sp.Transform.UserSettings()),
+            inversible=False,
             schema_preserving=False,
         )
     )
 
 
-def synthetic(
-    tables_sampling_ratios: t.Optional[t.Mapping[str, float]] = None
-) -> Transform:
+def automatic_user_settings() -> Transform:
+    """Transform to be called on a protected dataset
+    we want to change the schema. It creates a scalar
+    whose value explicits the new type of the schema"""
+    return Transform(
+        sp.Transform(
+            name='automatic_user_settings',
+            spec=sp.Transform.Spec(
+                automatic_user_settings=sp.Transform.AutomaticUserSettings()
+            ),
+            inversible=False,
+            schema_preserving=False,
+            properties={'creation_time': str(datetime.datetime.now())},
+        )
+    )
+
+
+def synthetic() -> Transform:
     """Synthetic transform"""
     return Transform(
         sp.Transform(
@@ -619,67 +629,45 @@ def synthetic(
             ),
             inversible=False,
             schema_preserving=True,
-            properties={'sampling_ratios': json.dumps(tables_sampling_ratios)}
-            if tables_sampling_ratios is not None
-            else {},
         )
     )
 
 
-def protect(
-    protected_entities: t.List[t.List[t.Any]],
-    public_entities: t.Optional[t.List[t.List[t.Any]]] = None,
-) -> Transform:
-    if public_entities is None:
-        public_entities = []
+def protect() -> Transform:
+    """Transform used for protection should be called on:
+    - the dataset that needs to be protected as the first arg
+    - a kwarg protected_paths: scalar specifying the paths
+     to the entities to protect
+    - a kwarg public_paths: scalar specifying the paths to
+    the public tables"""
     return Transform(
         sp.Transform(
             name='Dataset with Protected Entity',
-            spec=sp.Transform.Spec(
-                protect_dataset=sp.Transform.Protect(
-                    protected_entities=[
-                        path.protobuf() for path in paths(protected_entities)
-                    ],
-                    public_entities=[
-                        path.protobuf() for path in paths(public_entities)
-                    ],
-                )
-            ),
+            spec=sp.Transform.Spec(protect_dataset=sp.Transform.Protect()),
             inversible=True,
             schema_preserving=False,
         )
     )
 
 
-def transcode(
-    statistics: st.Statistics, data_type: st.Type, protected_path: st.Path
-) -> st.Transform:
+def transcode() -> st.Transform:
     return Transform(
         sp.Transform(
-            name='Transcoding for synthetic data',
-            spec=sp.Transform.Spec(
-                transcode=sp.Transform.Transcode(
-                    statistics=statistics.protobuf(),
-                    data_type=data_type.protobuf(),
-                    protected_path=protected_path.protobuf(),
-                )
-            ),
+            name='Transcode',
+            spec=sp.Transform.Spec(transcode=sp.Transform.Transcode()),
             inversible=True,
             schema_preserving=False,
         )
     )
 
 
-def inverse_transcode(transcode_transform: st.Transform) -> st.Transform:
+def inverse_transcode() -> st.Transform:
 
-    assert transcode_transform.protobuf().spec.HasField('transcode')
     return Transform(
         sp.Transform(
             name='Inverse Transcoding for synthetic data',
             spec=sp.Transform.Spec(
-                inverse_transcode=sp.Transform.InverseTranscode(
-                    transcode=transcode_transform.protobuf().spec.transcode
-                )
+                inverse_transcode=sp.Transform.InverseTranscode()
             ),
             inversible=True,
             schema_preserving=False,
@@ -687,6 +675,34 @@ def inverse_transcode(transcode_transform: st.Transform) -> st.Transform:
     )
 
 
+def automatic_protected_paths() -> st.Transform:
+    """Transform that should be called on the dataset
+    that needs to be protected, it creates a scalar whose
+    value will explicit the paths to protect"""
+    return Transform(
+        sp.Transform(
+            name='automatic_protected_paths',
+            spec=sp.Transform.Spec(
+                protected_paths=sp.Transform.ProtectedPaths()
+            ),
+            properties={'creation_time': str(datetime.datetime.now())},
+        )
+    )
+
+
+def automatic_public_paths() -> st.Transform:
+    """Transform that should be called on the dataset
+    that needs to be protected, it creates a scalar whose
+    value will explicit the paths to public entities"""
+    return Transform(
+        sp.Transform(
+            name='automatic_public_paths',
+            spec=sp.Transform.Spec(public_paths=sp.Transform.PublicPaths()),
+            properties={'creation_time': str(datetime.datetime.now())},
+        )
+    )
+
+
 def get_item(path: st.Path) -> st.Transform:
     return Transform(
         sp.Transform(
@@ -700,5 +716,153 @@ def get_item(path: st.Path) -> st.Transform:
     )
 
 
+def assign_budget() -> st.Transform:
+    """Transform to assign a given privacy budget to a dataset.
+    It is used to specify the budget to compute the attributes
+    size, bounds, marginals"""
+
+    return Transform(
+        sp.Transform(
+            name='budget_assignment',
+            spec=sp.Transform.Spec(assign_budget=sp.Transform.AssignBudget()),
+        )
+    )
+
+
+def automatic_budget() -> st.Transform:
+    """Transform to create a scalar specifying a budget
+    automatically from the dataset it is called on.
+    The rule to fix the budget is set in the corresponding
+    op.
+    """
+
+    return Transform(
+        sp.Transform(
+            name='automatic_budget',
+            spec=sp.Transform.Spec(
+                automatic_budget=sp.Transform.AutomaticBudget()
+            ),
+        )
+    )
+
+
+def attributes_budget() -> st.Transform:
+    """Transform to create a scalar specifying an
+     epsilon,delta budget for the DP attributes of a
+    dataset. It is called on a scalar specifying a
+    global budget for attributes+sd."""
+
+    return Transform(
+        sp.Transform(
+            name='attributes_budget',
+            spec=sp.Transform.Spec(
+                attribute_budget=sp.Transform.AttributesBudget()
+            ),
+        )
+    )
+
+
+def sd_budget() -> st.Transform:
+    """Transform to create a scalar specifying an
+     epsilon,delta budget for a synthetic dataset.
+    It should be called on another scalar that specifies
+    a global budget (SD+DP attributes)"""
+
+    return Transform(
+        sp.Transform(
+            name='sd_budget',
+            spec=sp.Transform.Spec(sd_budget=sp.Transform.SDBudget()),
+        )
+    )
+
+
+def sampling_ratios() -> st.Transform:
+    """Transform to create a scalar specifying the sampling ratio for
+    each table to synthetize from.
+    It should be called on the dataset to synthetize from."""
+    return Transform(
+        sp.Transform(
+            name='sampling_ratios',
+            spec=sp.Transform.Spec(
+                sampling_ratios=sp.Transform.SamplingRatios()
+            ),
+        )
+    )
+
+
+def derive_seed(random_int: int) -> st.Transform:
+    """Transform to derive a seed from a master seed"""
+    return Transform(
+        sp.Transform(
+            name='derive_seed',
+            spec=sp.Transform.Spec(
+                derive_seed=sp.Transform.DeriveSeed(random_integer=random_int)
+            ),
+        )
+    )
+
+
+def group_by_pe() -> st.Transform:
+    """Transform that allows to group fields
+    by protected entity value. This implies that
+    the dataset on which the transform is
+    applied should be PEP"""
+    return Transform(
+        sp.Transform(
+            name='group_by',
+            spec=sp.Transform.Spec(group_by_pe=sp.Transform.GroupByPE()),
+        )
+    )
+
+
+def select_sql(query: str) -> st.Transform:
+    """Transform that applies a query to a dataset.
+    Calling .schema() or .to_arrow() on a select_sql transformed dataset
+    the .sql method will be invoked and the query will be executed.
+    """
+    return Transform(
+        sp.Transform(
+            name='select_sql',
+            spec=sp.Transform.Spec(
+                select_sql=sp.Transform.SelectSql(query=query)
+            ),
+            inversible=False,
+            schema_preserving=False,
+        )
+    )
+
+
+def extract(
+    size: int,
+) -> st.Transform:
+    """Transform that should be called on a dataset from which we want to
+    extract some rows from according to the size parameter and a kwargs
+    random_seed, a scalar that is a seed. For now, seed and size are
+    ignored and iterating on the extract transfomed dataset will be as
+    iterating over the parent dataset.
+    """
+    return Transform(
+        sp.Transform(
+            name='extract',
+            spec=sp.Transform.Spec(extract=sp.Transform.Extract(size=size)),
+            inversible=False,
+            schema_preserving=True,
+        )
+    )
+
+
+def relationship_spec() -> st.Transform:
+    """Transform that allows to redefine the primary and foreign keys
+    of a dataset."""
+    return Transform(
+        sp.Transform(
+            name='relationship_spec',
+            spec=sp.Transform.Spec(
+                relationship_spec=sp.Transform.RelationshipSpec()
+            ),
+        )
+    )
+
+
 if t.TYPE_CHECKING:
     test_transform: st.Transform = Transform(sp.Transform())
